import type { ButtonProps } from '@blueprintjs/core'
import { Button, Dialog, DialogBody, DialogFooter } from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { memo, useCallback, useEffect, useState, type FC } from 'react'
import EmailTemplateFormContent from '../EmailTemplateFormContent'
import { addEmailTemplate, updateEmailTemplate } from '../indexeddb/operations'
import type { EmailTemplate } from '../indexeddb/types'

interface EmailTemplateFormDialogProps {
  template?: EmailTemplate
  emailAddressId: number
  buttonProps: ButtonProps
}

const EmailTemplateFormDialog: FC<EmailTemplateFormDialogProps> = ({
  template,
  emailAddressId,
  buttonProps,
}) => {
  const { notify } = useNotifyContext()
  const [isOpen, setIsOpen] = useState(false)

  const [context, setContext] = useState<string>('')
  const [content, setContent] = useState<string>('')
  const [selectedAddressId, setSelectedAddressId] = useState<number>(0)

  useEffect(() => {
    setContext(template?.context || '')
    setContent(template?.content || '')
    setSelectedAddressId(template?.emailAddressId || emailAddressId)
  }, [template, isOpen])

  const clearInput = useCallback(() => {
    setContext('')
    setContent('')
  }, [])

  const handleAddButton = useCallback(
    async (emailTemplate: Omit<EmailTemplate, 'id'>) => {
      try {
        await addEmailTemplate(emailTemplate)
        clearInput()
        setIsOpen(false)
      } catch (err) {
        notify(`Failed to add template '${emailTemplate.context}': ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify]
  )

  const handleUpdateButton = useCallback(
    async (emailTemplate: EmailTemplate) => {
      try {
        await updateEmailTemplate(emailTemplate)
        setIsOpen(false)
      } catch (err) {
        notify(`Failed to update template '${emailTemplate.context}': ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify]
  )

  return (
    <>
      <Button {...buttonProps} onClick={() => setIsOpen(true)} />
      <Dialog
        isOpen={isOpen}
        onClose={() => setIsOpen(false)}
        icon={template ? 'edit' : 'plus'}
        title={template ? 'Edit template' : 'New template'}
      >
        <DialogBody>
          <EmailTemplateFormContent
            context={context}
            content={content}
            emailAddressId={selectedAddressId}
            onContextChange={(value: string) => setContext(value)}
            onContentChange={(value: string) => setContent(value)}
            onEmailAddressIdChange={(value: number) =>
              setSelectedAddressId(value)
            }
          />
        </DialogBody>
        <DialogFooter
          actions={
            template ? (
              <Button
                disabled={!context || !selectedAddressId}
                onClick={() =>
                  handleUpdateButton({
                    id: template.id,
                    context,
                    content,
                    emailAddressId: selectedAddressId,
                  })
                }
                intent='primary'
                icon='edit'
                text='Save changes'
              />
            ) : (
              <Button
                disabled={!context || !selectedAddressId}
                onClick={() =>
                  handleAddButton({
                    context,
                    content,
                    emailAddressId: selectedAddressId,
                  })
                }
                intent='primary'
                icon='plus'
                text='Add'
              />
            )
          }
        />
      </Dialog>
    </>
  )
}

export default memo(EmailTemplateFormDialog)
