import {
  Button,
  InputGroup,
  Label,
  NumericInput,
  TextArea,
} from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useLiveQuery } from 'dexie-react-hooks'
import { memo, useCallback, useEffect, useState, type FC } from 'react'
import EmailAddressSelector from '../EmailAddressSelector'
import {
  addEmailInject,
  getEmailInjectByInjectInfoId,
  updateEmailInject,
} from '../indexeddb/operations'
import type { EmailInject } from '../indexeddb/types'

interface EmailInjectFormProps {
  injectInfoId: number
}

const EmailInjectForm: FC<EmailInjectFormProps> = ({ injectInfoId }) => {
  const emailInject = useLiveQuery(
    () => getEmailInjectByInjectInfoId(injectInfoId),
    [injectInfoId],
    null
  ) as EmailInject

  const { notify } = useNotifyContext()

  const [subject, setSubject] = useState<string>('')
  const [content, setContent] = useState<string>('')
  const [selectedAddressId, setSelectedAddressId] = useState<number>(0)
  const [extraCopies, setExtraCopies] = useState<number>(0)

  useEffect(() => {
    setSubject(emailInject?.subject || '')
    setContent(emailInject?.content || '')
    setSelectedAddressId(emailInject?.emailAddressId || 0)
    setExtraCopies(emailInject?.extraCopies || 0)
  }, [emailInject])

  const handleUpdateButton = useCallback(
    async (newEmailInject: EmailInject | Omit<EmailInject, 'id'>) => {
      try {
        if (emailInject) {
          await updateEmailInject({ id: emailInject.id, ...newEmailInject })
        } else {
          await addEmailInject(newEmailInject)
        }
      } catch (err) {
        notify(`Failed to update email inject: ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify, emailInject]
  )

  return (
    <div>
      <EmailAddressSelector
        emailAddressId={selectedAddressId}
        onChange={id => setSelectedAddressId(id)}
      />
      <Label>
        Subject
        <InputGroup
          placeholder='Input text'
          value={subject}
          onChange={e => setSubject(e.target.value)}
        />
      </Label>
      <Label>
        Content
        <TextArea
          value={content}
          style={{
            width: '100%',
            height: '10rem',
            resize: 'none',
            overflowY: 'auto',
          }}
          placeholder='Input text'
          onChange={e => setContent(e.target.value)}
        />
      </Label>
      <Label>
        Extra copies
        <NumericInput
          placeholder='Input number'
          min={0}
          value={extraCopies}
          onValueChange={(value: number) => setExtraCopies(value)}
        />
      </Label>
      <Button
        onClick={() =>
          handleUpdateButton({
            injectInfoId,
            subject,
            content,
            emailAddressId: selectedAddressId,
            extraCopies,
          })
        }
        intent='primary'
        icon='edit'
        text='Save changes'
      />
    </div>
  )
}

export default memo(EmailInjectForm)
