import { Button, InputGroup, Label } from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useLiveQuery } from 'dexie-react-hooks'
import { memo, useCallback, useEffect, useState, type FC } from 'react'
import {
  addQuestionnaire,
  getQuestionnaireByInjectInfoId,
  updateQuestionnaire,
} from '../../indexeddb/operations'
import type { Questionnaire } from '../../indexeddb/types'
import QuestionnaireQuestionForm from './QuestionnaireQuestionForm'
import QuestionnaireQuestions from './QuestionnaireQuestions'

interface QuestionnaireFormProps {
  injectInfoId: number
}

const QuestionnaireForm: FC<QuestionnaireFormProps> = ({ injectInfoId }) => {
  const questionnaire = useLiveQuery(
    () => getQuestionnaireByInjectInfoId(injectInfoId),
    [injectInfoId],
    null
  ) as Questionnaire

  const { notify } = useNotifyContext()

  const [title, setTitle] = useState<string>('')

  useEffect(() => {
    setTitle(questionnaire?.title || '')

    if (questionnaire === undefined) {
      addQuestionnaire({ injectInfoId, title: '' })
    }
  }, [questionnaire, injectInfoId])

  const handleUpdateButton = useCallback(
    async (newQuestionnaire: Questionnaire | Omit<Questionnaire, 'id'>) => {
      try {
        if (questionnaire) {
          await updateQuestionnaire({
            id: questionnaire.id,
            ...newQuestionnaire,
          })
        } else {
          await addQuestionnaire(newQuestionnaire)
        }
      } catch (err) {
        notify(`Failed to update questionnaire: ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify, questionnaire]
  )

  return (
    <div>
      <Label>
        Title
        <InputGroup
          placeholder='Input text'
          value={title}
          onChange={e => setTitle(e.target.value)}
        />
      </Label>
      {questionnaire && (
        <>
          <QuestionnaireQuestions questionnaireId={questionnaire.id} />
          <QuestionnaireQuestionForm
            questionnaireId={questionnaire.id}
            buttonProps={{
              minimal: true,
              text: 'Add new question',
              alignText: 'left',
              icon: 'plus',
              style: { padding: '1rem', width: '100%', marginBottom: '1rem' },
            }}
          />
        </>
      )}
      <Button
        onClick={() =>
          handleUpdateButton({
            injectInfoId,
            title,
          })
        }
        intent='primary'
        icon='edit'
        text='Save changes'
      />
    </div>
  )
}

export default memo(QuestionnaireForm)
