import { Button } from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useLiveQuery } from 'dexie-react-hooks'
import { memo, useCallback, useEffect, useState, type FC } from 'react'
import ToolResponseFormContent from '../ToolResponseFormContent'
import {
  addToolResponse,
  getToolResponseByActivityId,
  updateToolResponse,
} from '../indexeddb/operations'
import type { ToolResponse } from '../indexeddb/types'

interface ToolResponseFormProps {
  learningActivityId: number
}

const ToolResponseForm: FC<ToolResponseFormProps> = ({
  learningActivityId,
}) => {
  const response = useLiveQuery(
    () => getToolResponseByActivityId(learningActivityId),
    [learningActivityId],
    null
  ) as ToolResponse

  const { notify } = useNotifyContext()

  const [parameter, setParameter] = useState<string>('')
  const [content, setContent] = useState<string>('')
  const [isRegex, setIsRegex] = useState<boolean>(false)
  const [selectedToolId, setSelectedToolId] = useState<number>(0)

  useEffect(() => {
    setParameter(response?.parameter || '')
    setContent(response?.content || '')
    setIsRegex(response?.isRegex || false)
    setSelectedToolId(response?.toolId || 0)
  }, [response])

  const handleUpdateButton = useCallback(
    async (newResponse: ToolResponse | Omit<ToolResponse, 'id'>) => {
      try {
        if (response) {
          await updateToolResponse({ id: response.id, ...newResponse })
        } else {
          await addToolResponse(newResponse)
        }
      } catch (err) {
        notify(`Failed to update tool response: ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify, response]
  )

  return (
    <div>
      <ToolResponseFormContent
        parameter={parameter}
        content={content}
        isRegex={isRegex}
        toolId={selectedToolId}
        onParameterChange={(value: string) => setParameter(value)}
        onContentChange={(value: string) => setContent(value)}
        onIsRegexChange={(value: boolean) => setIsRegex(value)}
        onToolIdChange={(value: number) => setSelectedToolId(value)}
      />
      <Button
        disabled={!parameter || !selectedToolId}
        onClick={() =>
          handleUpdateButton({
            learningActivityId,
            parameter,
            content,
            isRegex,
            toolId: selectedToolId,
          })
        }
        intent='primary'
        icon='edit'
        text='Save changes'
      />
    </div>
  )
}

export default memo(ToolResponseForm)
